

*            *--------------------------------------------------------
*            *                                                       |
*            * Sobolev Constrained Estimation of Option Pricing      |
*            *                                                       |
*            *--------------------------------------------------------

$OFFSYMXREF  OFFSYMLIST  OFFUELLIST  OFFUELXREF
 OPTION NLP=Minos5, seed=666, OPTCR=.0000001,
        ITERLIM=100000, LIMCOL=0, LIMROW=0, RESLIM=100000;

*Seeds that work well 5123, 666

*                           SET DEFINITION
*                           ______________
   Set j   strikes   /1*25/
    jLo(j) subset    /1*20/
    jHi(j) subset    /23*25/;
   Alias (j,jj);

   Set i   obs      /1*500/;
   Alias (i,ii);

$Include Representors.txt


*                           Parameter Specification
*                           _______________________

    Scalar nobs          number of observations /500/;
    Scalar Lhat          estimator of Sobolev bound;

    Parameter distinctXs(j) distinct strike prices;
              distinctXs(j)  = Splusdatj(j,'distinctXs');

    Parameter distinctHs(j)  distinct call function values;
              distinctHs(j) = SplusDatj(j,'distinctHs')/1000.;

    Parameter X(i)          strike price;
              X(i)        = Splusdati(i,'X');

    Parameter Xid(i)        strike price identifier;
              Xid(i)        = Splusdati(i,'Xid');

    Parameter H(i)          call function;
              H(i)        = SplusDati(i,'H')/1000.;



    Parameter e(i)          Residuals;
    e(i)                  = Normal(0,1);

    Parameter y(i)          Dependent Variable;
              y(i)        = SplusDati(i,'y')/1000;
*     y(i)                 = H(i)
    y(i)                  = H(i) + .1*e(i)*H(i);
*    y(i)                  = H(i) + .00001*e(i)*H(i);


    Parameter yder0(j)      Estimated Function;
    Parameter yder1(j)      Estimated Der 1;
    Parameter yder2(j)      Estimated Der 2;

    Set ij(i, j);
    ij(i, j)$(Ord(j) = Xid(i)) = Yes;
*    ij(i, j)$(distinctXs(j) = X(i)) = Yes;
    Display ij;     Display y;



*                           Variable Definitions
*                           ____________________

    Variables Obj, yhat(j), c(j);
*    c.l(j)=0.;   c.lo(j)=-10.;  c.up(j)=10.;

*                           Equation Definitions
*                           ____________________
    Equations Min;
*   Min.. Obj =E= Sum(ij(i, j), sqr( y(i) - yhat(j)))/nobs;
    Min.. Obj =E= Sum(ij(i, j), sqr(   (y(i) - yhat(j))/H(i))  )/nobs;

    Equations Yyhat;
    Yyhat(j).. yhat(j) =E= sum(jj,R(j,jj)*c(jj));

    Equations SobCon;
    SobCon.. sum(j,yhat(j)*c(j)) =L=  1.7;
*    SobCon.. sum(j,yhat(j)*c(j)) =L=  1.812;
*True value is about 1.812


    Equations Monotone;
    Monotone(j).. yhat(j) =G= yhat(j+1);

    Equations Convex;
    Convex(j).. yhat(j+2) - 2*yhat(j+1) + yhat(j) =G= 0.;

*    Equations Unimodal;
*    Unimodal(jLo).. yhat(jLo+3) - 3*yhat(jLo+2) + 3*yhat(jLo+1) - yhat(jLo) =g= 0.;

*    Equations Unimodal;
*    Unimodal(jLo).. (yhat(jLo) - 2*yhat(jLo-1) + yhat(jLo-2))
*                      - (yhat(jLo-1) - 2*yhat(jLo-2) + yhat(jLo-3))   =G= 0.;

    Equations Unimodal;
    Unimodal(jLo).. sum(jj,Rder2(jj,jLo)*c(jj))-sum(jj,Rder2(jj,jLo-1)*c(jj)) =G= 0;

    Equations Yyder1;
    Yyder1(j).. sum(jj,Rder1(jj,j)*c(jj)) =L= 0;

    Equations Yyder2;
    Yyder2(j).. sum(jj,Rder2(jj,j)*c(jj)) =G= 0;

    Equations Yyder3Lo;
    Yyder3Lo(jLo).. sum(j,Rder3(j,jLo)*c(j)) =G= 0;

    Equations Yyder3Hi;
    Yyder3Hi(jHi).. sum(j,Rder3(j,jHi)*c(j)) =L= 0;

    Equations BoundA;
    BoundA.. sum(j,c(j)*Rder2(j,'1'))  =l= .0001;

    Equations BoundB;
    BoundB.. sum(j,c(j)*Rder2(j,'25')) =l= .0001;


*                           Model Definitions
*                           _________________

    Model Unr /Min,Yyhat,SobCon/;
    Model Res   /Min,Yyhat,Sobcon,Yyder1,Yyder2,Yyder3Lo,Yyder3Hi/;
*    Model Res   /Min,Yyhat,Sobcon,Yyder1,Yyder2/;
*    Model Res   /Min,Yyhat,Sobcon/;
*    Model Res /Min,Yyhat,Unimodal/;
*    Model Res   /Min,Yyhat,Monotone,Convex/;
*    Model Res   /Min,Yyhat,Sobcon,Yyder1,Yyder2/;
*    Model Res   /Min,Yyhat,Sobcon,Yyder1,Yyder2,Yyder3Lo,Yyder3Hi,Monotone,Convex/;


*                           Output File Definitions
*                           _______________________
*    File TrueDGM;            _______________________
    File Figures6_7ABC;
    File Con; Con.PS=1;
*   Con is the console



*                           Solve
*                           ______

    Solve Unr   minimizing Obj using NLP;
    Solve Res   minimizing Obj using NLP;
    Display y, yhat.l, c.l;

*   Calculate function and derivatives (note rescaling is now reversed)
    yder0(j) = sum(jj,    R(j,jj)*c.l(jj)) * 1000;
    yder1(j) = sum(jj,Rder1(jj,j)*c.l(jj)) * 10;
    yder2(j) = sum(jj,Rder2(jj,j)*c.l(jj)) / 10;


*    Put TrueDGM;
    Put Figures6_7ABC;
    Put @14 'X' @26 'yder0' @40 'yder1' @55 'yder2'/;
    Loop (j, put distinctXs(j):15:6 @15 yder0(j) :15:6 @30 yder1(j) :15:6 @45  yder2(j) :15:6 @60 /);

    Lhat =  sum(j,yhat.l(j)*c.l(j));  display Lhat;